@@ -14,8 +14,8 @@ Shelf plugin and control implementation
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: ed_shelf.py 62460 2009-10-21 03:13:18Z CJP $"
-__revision__ = "$Revision: 62460 $"
+__svnid__ = "$Id: ed_shelf.py 63523 2010-02-19 15:57:32Z CJP $"
+__revision__ = "$Revision: 63523 $"
 
 #-----------------------------------------------------------------------------#
 # Imports
@@ -28,7 +28,7 @@ import ed_glob
 from profiler import Profile_Get
 import plugin
 import iface
-import extern.flatnotebook as FNB
+import extern.aui as aui
 
 #--------------------------------------------------------------------------#
 # Globals
@@ -133,17 +133,18 @@ class Shelf(plugin.Plugin):
 
 #--------------------------------------------------------------------------#
 
-class EdShelfBook(FNB.FlatNotebook):
+class EdShelfBook(aui.AuiNotebook):
     """Shelf notebook control"""
     def __init__(self, parent):
-        FNB.FlatNotebook.__init__(self, parent,
-                                  style=FNB.FNB_FF2 |
-                                        FNB.FNB_X_ON_TAB |
-                                        FNB.FNB_BACKGROUND_GRADIENT |
-                                        FNB.FNB_NODRAG |
-                                        FNB.FNB_BOTTOM |
-                                        FNB.FNB_NO_X_BUTTON |
-                                        FNB.FNB_MOUSE_MIDDLE_CLOSES_TABS)
+        style = aui.AUI_NB_BOTTOM | \
+                aui.AUI_NB_TAB_SPLIT | \
+                aui.AUI_NB_SCROLL_BUTTONS | \
+                aui.AUI_NB_CLOSE_ON_ACTIVE_TAB | \
+                aui.AUI_NB_TAB_MOVE | \
+                aui.AUI_NB_DRAW_DND_TAB
+        if wx.Platform == '__WXMAC__':
+            style |= aui.AUI_NB_CLOSE_ON_TAB_LEFT
+        aui.AuiNotebook.__init__(self, parent, style=style)
 
         # Attributes
         self._parent = parent
@@ -153,6 +154,7 @@ class EdShelfBook(FNB.FlatNotebook):
 
         # Setup
         self.SetImageList(self._imglst)
+        self.SetSashDClickUnsplit(True)
 
     @property
     def ImgIdx(self):
@@ -171,7 +173,9 @@ class EdShelfBook(FNB.FlatNotebook):
         @param name: Items name used for page text in notebook
 
         """
-        self.AddPage(item, u"%s - %d" % (name, self._open.get(name, 0)))
+        self.AddPage(item, 
+                     u"%s - %d" % (name, self._open.get(name, 0)),
+                     select=True)
 
         # Set the tab icon
         if imgid >= 0 and Profile_Get('TABICONS', default=True):
@@ -199,10 +203,17 @@ class EdShelfBook(FNB.FlatNotebook):
         """
         count = 0
         for page in xrange(self.GetPageCount()):
-            if item_name == re.sub(PGNUM_PAT, u'', self.GetPageText(page), 1):
+            if self.GetPageText(page).startswith(item_name):
                 count = count + 1
         return count
 
+    def GetMainWindow(self):
+        """Get the main window that this shelf instance was created for
+        @return: ed_main.MainWindow
+
+        """
+        return self._parent
+
     def GetOpen(self):
         """Get the list of open shelf items
         @return: list
@@ -365,7 +376,7 @@ class EdShelfDelegate(object):
         @return: reference to ed_main.MainWindow or None
 
         """
-        return self._shelf.GetParent()
+        return self._shelf.GetMainWindow()
 
     def GetWindow(self):
         """Return reference to the Shelfs window component
