@@ -12,14 +12,13 @@ This file contains various helper functions and utilities that the program uses.
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: util.py 61179 2009-06-23 22:47:20Z CJP $"
-__revision__ = "$Revision: 61179 $"
+__svnid__ = "$Id: util.py 63289 2010-01-28 03:10:42Z CJP $"
+__revision__ = "$Revision: 63289 $"
 
 #--------------------------------------------------------------------------#
 # Imports
 import os
 import sys
-import types
 import mimetypes
 import encodings
 import codecs
@@ -33,8 +32,6 @@ import ed_crypt
 import dev_tool
 import syntax.syntax as syntax
 import syntax.synglob as synglob
-import eclib # TODO: Temporary till refs to HexToRGB and AdjustColor are
-             #       no longer used anywhere
 import ebmlib
 
 _ = wx.GetTranslation
@@ -52,11 +49,15 @@ class DropTargetFT(wx.PyDropTarget):
 
         """
         wx.PyDropTarget.__init__(self)
+
+        # Attributes
         self.window = window
         self._data = dict(data=None, fdata=None, tdata=None,
                           tcallb=textcallback, fcallb=filecallback)
         self._tmp = None
         self._lastp = None
+
+        # Setup
         self.InitObjects()
 
     def CreateDragString(self, txt):
@@ -234,6 +235,10 @@ class DropTargetFT(wx.PyDropTarget):
 #---- End FileDropTarget ----#
 
 class EdClipboard(ebmlib.CycleCache):
+    """Local clipboard object
+    @todo: make into a singleton
+
+    """
     def GetNext(self):
         """Get the next item in the cache"""
         # Initialize the clipboard if it hasn't been loaded yet and
@@ -286,7 +291,8 @@ def GetClipboardText(primary=False):
 
     text_obj = wx.TextDataObject()
     rtxt = None
-    if wx.TheClipboard.Open():
+    
+    if wx.TheClipboard.IsOpened() or wx.TheClipboard.Open():
         if wx.TheClipboard.GetData(text_obj):
             rtxt = text_obj.GetText()
         wx.TheClipboard.Close()
@@ -314,7 +320,7 @@ def SetClipboardText(txt, primary=False):
 
     data_o = wx.TextDataObject()
     data_o.SetText(txt)
-    if wx.TheClipboard.Open():
+    if wx.TheClipboard.IsOpened() or wx.TheClipboard.Open():
         wx.TheClipboard.SetData(data_o)
         wx.TheClipboard.Close()
         if primary and wx.Platform == '__WXGTK__':
@@ -526,6 +532,15 @@ def ResolvConfigDir(config_dir, sys_only=False):
         if os.path.exists(user_config):
             return user_config + os.sep
 
+    # Check if the system install path has already been resolved once before
+    if ed_glob.CONFIG['INSTALL_DIR'] != u"":
+        tmp = os.path.join(ed_glob.CONFIG['INSTALL_DIR'], config_dir)
+        tmp = os.path.normpath(tmp) + os.sep
+        if os.path.exists(tmp):
+            return tmp
+        else:
+            del tmp
+
     # The following lines are used only when Editra is being run as a
     # source package. If the found path does not exist then Editra is
     # running as as a built package.
@@ -534,14 +549,14 @@ def ResolvConfigDir(config_dir, sys_only=False):
         path = os.sep.join(path.split(os.sep)[:-2])
         path =  path + os.sep + config_dir + os.sep
         if os.path.exists(path):
-            if not isinstance(path, types.UnicodeType):
+            if not ebmlib.IsUnicode(path):
                 path = unicode(path, sys.getfilesystemencoding())
             return path
 
     # If we get here we need to do some platform dependant lookup
     # to find everything.
     path = sys.argv[0]
-    if not isinstance(path, types.UnicodeType):
+    if not ebmlib.IsUnicode(path):
         path = unicode(path, sys.getfilesystemencoding())
 
     # If it is a link get the real path
@@ -586,7 +601,7 @@ def ResolvConfigDir(config_dir, sys_only=False):
     path = os.path.normpath(pro_path) + os.sep
 
     # Make sure path is unicode
-    if not isinstance(path, types.UnicodeType):
+    if not ebmlib.IsUnicode(path):
         path = unicode(path, sys.getdefaultencoding())
 
     return path
